@@ -15,13 +15,16 @@ use File::Basename;
 use File::Which qw(which);
 use IPC::Run qw(run);
 use Cwd;
+use Config;
+use IPC::Open3;
+use Symbol 'gensym';
 
-our $VERSION = "0.23";
+our $VERSION = "0.24";
 
 ###########################################
 sub new {
 ###########################################
-    my($class, %options) = @_;
+    my ( $class, %options ) = @_;
 
     my $self = {
         tar                   => undef,
@@ -36,29 +39,34 @@ sub new {
         %options,
     };
 
+    if ( ( $Config{osname} eq 'openbsd' ) and ( $self->{tar_read_options} ) ) {
+        $self->{tar_read_options} = '-' . $self->{tar_read_options};
+    }
+
     bless $self, $class;
 
-    $self->{tar} = which("tar") unless defined $self->{tar};
+    $self->{tar} = which("tar")  unless defined $self->{tar};
     $self->{tar} = which("gtar") unless defined $self->{tar};
 
-    if( ! defined $self->{tar} ) {
+    unless ( defined $self->{tar} ) {
         LOGDIE "tar not found in PATH, please specify location";
     }
 
-    if(defined $self->{ramdisk}) {
+    if ( defined $self->{ramdisk} ) {
         my $rc = $self->ramdisk_mount( %{ $self->{ramdisk} } );
-        if(!$rc) {
+        unless ($rc) {
             LOGDIE "Mounting ramdisk failed";
         }
         $self->{tmpdir} = $self->{ramdisk}->{tmpdir};
-    } else {
-        $self->{tmpdir} = tempdir($self->{tmpdir} ? 
-                                        (DIR => $self->{tmpdir}) : ());
+    }
+    else {
+        $self->{tmpdir} =
+          tempdir( $self->{tmpdir} ? ( DIR => $self->{tmpdir} ) : () );
     }
 
-    $self->{tardir} = File::Spec->catfile($self->{tmpdir}, "tar");
-    mkpath [$self->{tardir}], 0, 0755 or
-        LOGDIE "Cannot mkpath $self->{tardir} ($!)";
+    $self->{tardir} = File::Spec->catfile( $self->{tmpdir}, "tar" );
+    mkpath [ $self->{tardir} ], 0, 0755
+      or LOGDIE "Cannot mkpath $self->{tardir} ($!)";
 
     $self->{objdir} = tempdir();
 
@@ -68,7 +76,7 @@ sub new {
 ###########################################
 sub tardir {
 ###########################################
-    my($self) = @_;
+    my ($self) = @_;
 
     return $self->{tardir};
 }
@@ -76,32 +84,53 @@ sub tardir {
 ###########################################
 sub read {
 ###########################################
-    my($self, $tarfile, @files) = @_;
-
+    my ( $self, $tarfile, @files ) = @_;
     my $cwd = getcwd();
 
-    unless(File::Spec::Functions::file_name_is_absolute($tarfile)) {
-        $tarfile = File::Spec::Functions::rel2abs($tarfile, $cwd);
+    unless ( File::Spec::Functions::file_name_is_absolute($tarfile) ) {
+        $tarfile = File::Spec::Functions::rel2abs( $tarfile, $cwd );
     }
 
-    chdir $self->{tardir} or 
-        LOGDIE "Cannot chdir to $self->{tardir}";
+    chdir $self->{tardir}
+      or LOGDIE "Cannot chdir to $self->{tardir}";
 
-    my $compr_opt = "";
+    my $compr_opt = '';
     $compr_opt = $self->is_compressed($tarfile);
 
-    my $cmd = [$self->{tar}, "${compr_opt}x$self->{tar_read_options}",
-               @{$self->{tar_gnu_read_options}},
-               "-f", $tarfile, @files];
+    # actually, prepending '-' would work with any modern GNU tar
+    if ( $Config{osname} eq 'openbsd' ) {
+        $compr_opt = '-' . $compr_opt if ($compr_opt);
+    }
 
-    DEBUG "Running @$cmd";
+    my @cmd;
 
-    my $rc = run($cmd, \my($in, $out, $err));
+    if ( $Config{osname} eq 'openbsd' ) {
+        push( @cmd, $self->{tar} );
+        push( @cmd, $compr_opt ) if ( $compr_opt ne '' );
+        push( @cmd, '-x' );
+        push( @cmd, $self->{tar_read_options} )
+          if ( $self->{tar_read_options} ne '' );
+        push( @cmd, @{ $self->{tar_gnu_read_options} } )
+          if ( scalar( @{ $self->{tar_gnu_read_options} } ) > 0 );
+        push( @cmd, '-f', $tarfile, @files );
+    }
+    else {
+        @cmd = (
+            $self->{tar},
+            "${compr_opt}x$self->{tar_read_options}",
+            @{ $self->{tar_gnu_read_options} },
+            '-f', $tarfile, @files
+        );
+    }
 
-    if(!$rc) {
-         ERROR "@$cmd failed: $err";
-         chdir $cwd or LOGDIE "Cannot chdir to $cwd";
-         return undef;
+    DEBUG "Running @cmd";
+
+    my $error_code = run( \@cmd, \my ( $in, $out, $err ) );
+
+    unless ($error_code) {
+        ERROR "@cmd failed: $err";
+        chdir $cwd or LOGDIE "Cannot chdir to $cwd";
+        return undef;
     }
 
     WARN $err if $err;
@@ -114,19 +143,19 @@ sub read {
 ###########################################
 sub is_compressed {
 ###########################################
-    my($self, $tarfile) = @_;
+    my ( $self, $tarfile ) = @_;
 
     return 'z' if $tarfile =~ /\.t?gz$/i;
     return 'j' if $tarfile =~ /\.bz2$/i;
 
-        # Sloppy check for gzip files
+    # Sloppy check for gzip files
     open FILE, "<$tarfile" or die "Cannot open $tarfile";
     binmode FILE;
-    my $read = sysread(FILE, my $two, 2, 0) or die "Cannot sysread";
+    my $read = sysread( FILE, my $two, 2, 0 ) or die "Cannot sysread";
     close FILE;
-    return 'z' if 
-        ord(substr($two, 0, 1)) eq 0x1F and 
-        ord(substr($two, 1, 1)) eq 0x8B;
+    return 'z'
+      if ord( substr( $two, 0, 1 ) ) eq 0x1F
+      and ord( substr( $two, 1, 1 ) ) eq 0x8B;
 
     return q{};
 }
@@ -134,11 +163,11 @@ sub is_compressed {
 ###########################################
 sub locate {
 ###########################################
-    my($self, $rel_path) = @_;
+    my ( $self, $rel_path ) = @_;
 
-    my $real_path = File::Spec->catfile($self->{tardir}, $rel_path);
+    my $real_path = File::Spec->catfile( $self->{tardir}, $rel_path );
 
-    if(-e $real_path) {
+    if ( -e $real_path ) {
         DEBUG "$real_path exists";
         return $real_path;
     }
@@ -151,66 +180,71 @@ sub locate {
 ###########################################
 sub add {
 ###########################################
-    my($self, $rel_path, $path_or_stringref, $opts) = @_;
-            
-    if($opts) {
-        if(!ref($opts) or ref($opts) ne 'HASH') {
+    my ( $self, $rel_path, $path_or_stringref, $opts ) = @_;
+
+    if ($opts) {
+        if ( !ref($opts) or ref($opts) ne 'HASH' ) {
             LOGDIE "Option parameter given to add() not a hashref.";
         }
     }
 
-    my $perm    = $opts->{perm} if defined $opts->{perm};
-    my $uid     = $opts->{uid} if defined $opts->{uid};
-    my $gid     = $opts->{gid} if defined $opts->{gid};
+    my $perm    = $opts->{perm}    if defined $opts->{perm};
+    my $uid     = $opts->{uid}     if defined $opts->{uid};
+    my $gid     = $opts->{gid}     if defined $opts->{gid};
     my $binmode = $opts->{binmode} if defined $opts->{binmode};
 
-    my $target = File::Spec->catfile($self->{tardir}, $rel_path);
+    my $target = File::Spec->catfile( $self->{tardir}, $rel_path );
     my $target_dir = dirname($target);
 
-    if( ! -d $target_dir ) {
-        if( ref($path_or_stringref) ) {
-            $self->add( dirname( $rel_path ), dirname( $target_dir ) );
-        } else {
-            $self->add( dirname( $rel_path ), dirname( $path_or_stringref ) );
+    if ( !-d $target_dir ) {
+        if ( ref($path_or_stringref) ) {
+            $self->add( dirname($rel_path), dirname($target_dir) );
+        }
+        else {
+            $self->add( dirname($rel_path), dirname($path_or_stringref) );
         }
     }
 
-    if(ref($path_or_stringref)) {
+    if ( ref($path_or_stringref) ) {
         open FILE, ">$target" or LOGDIE "Can't open $target ($!)";
-        if(defined $binmode) {
+        if ( defined $binmode ) {
             binmode FILE, $binmode;
         }
         print FILE $$path_or_stringref;
         close FILE;
-    } elsif( -d $path_or_stringref ) {
-          # perms will be fixed further down
-        mkpath($target, 0, 0755) unless -d $target;
-    } else {
-        copy $path_or_stringref, $target or
-            LOGDIE "Can't copy $path_or_stringref to $target ($!)";
+    }
+    elsif ( -d $path_or_stringref ) {
+
+        # perms will be fixed further down
+        mkpath( $target, 0, 0755 ) unless -d $target;
+    }
+    else {
+        copy $path_or_stringref, $target
+          or LOGDIE "Can't copy $path_or_stringref to $target ($!)";
     }
 
-    if(defined $uid) {
-        chown $uid, -1, $target or
-            LOGDIE "Can't chown $target uid to $uid ($!)";
+    if ( defined $uid ) {
+        chown $uid, -1, $target
+          or LOGDIE "Can't chown $target uid to $uid ($!)";
     }
 
-    if(defined $gid) {
-        chown -1, $gid, $target or
-            LOGDIE "Can't chown $target gid to $gid ($!)";
+    if ( defined $gid ) {
+        chown -1, $gid, $target
+          or LOGDIE "Can't chown $target gid to $gid ($!)";
     }
 
-    if(defined $perm) {
-        chmod $perm, $target or 
-                LOGDIE "Can't chmod $target to $perm ($!)";
+    if ( defined $perm ) {
+        chmod $perm, $target
+          or LOGDIE "Can't chmod $target to $perm ($!)";
     }
 
-    if(!defined $uid and 
-       !defined $gid and 
-       !defined $perm and
-       !ref($path_or_stringref)) {
-        perm_cp($path_or_stringref, $target) or
-            LOGDIE "Can't perm_cp $path_or_stringref to $target ($!)";
+    if (    !defined $uid
+        and !defined $gid
+        and !defined $perm
+        and !ref($path_or_stringref) )
+    {
+        perm_cp( $path_or_stringref, $target )
+          or LOGDIE "Can't perm_cp $path_or_stringref to $target ($!)";
     }
 
     1;
@@ -219,22 +253,22 @@ sub add {
 ######################################
 sub perm_cp {
 ######################################
-    my($source, $target) = @_;
+    my ( $source, $target ) = @_;
 
     # Lifted from Ben Okopnik's
     # http://www.linuxgazette.com/issue87/misc/tips/cpmod.pl.txt
 
     my $perms = perm_get($source);
-    perm_set($target, $perms);
+    perm_set( $target, $perms );
 }
 
 ######################################
 sub perm_get {
 ######################################
-    my($filename) = @_;
+    my ($filename) = @_;
 
-    my @stats = (stat $filename)[2,4,5] or
-        LOGDIE "Cannot stat $filename ($!)";
+    my @stats = ( stat $filename )[ 2, 4, 5 ]
+      or LOGDIE "Cannot stat $filename ($!)";
 
     return \@stats;
 }
@@ -242,22 +276,22 @@ sub perm_get {
 ######################################
 sub perm_set {
 ######################################
-    my($filename, $perms) = @_;
+    my ( $filename, $perms ) = @_;
 
-      # ignore errors here, as we can't change uid/gid unless we're
-      # the superuser (see LIMITATIONS section)
-    chown($perms->[1], $perms->[2], $filename);
+    # ignore errors here, as we can't change uid/gid unless we're
+    # the superuser (see LIMITATIONS section)
+    chown( $perms->[1], $perms->[2], $filename );
 
-    chmod($perms->[0] & 07777,    $filename) or
-        LOGDIE "Cannot chmod $filename ($!)";
+    chmod( $perms->[0] & 07777, $filename )
+      or LOGDIE "Cannot chmod $filename ($!)";
 }
 
 ###########################################
 sub remove {
 ###########################################
-    my($self, $rel_path) = @_;
+    my ( $self, $rel_path ) = @_;
 
-    my $target = File::Spec->catfile($self->{tardir}, $rel_path);
+    my $target = File::Spec->catfile( $self->{tardir}, $rel_path );
 
     rmtree($target) or LOGDIE "Can't rmtree $target ($!)";
 }
@@ -265,13 +299,13 @@ sub remove {
 ###########################################
 sub list_all {
 ###########################################
-    my($self) = @_;
+    my ($self) = @_;
 
     my @entries = ();
 
     $self->list_reset();
 
-    while(my $entry = $self->list_next()) {
+    while ( my $entry = $self->list_next() ) {
         push @entries, $entry;
     }
 
@@ -281,23 +315,27 @@ sub list_all {
 ###########################################
 sub list_reset {
 ###########################################
-    my($self) = @_;
+    my ($self) = @_;
 
-    my $list_file = File::Spec->catfile($self->{objdir}, "list");
+    my $list_file = File::Spec->catfile( $self->{objdir}, "list" );
     open FILE, ">$list_file" or LOGDIE "Can't open $list_file";
 
     my $cwd = getcwd();
     chdir $self->{tardir} or LOGDIE "Can't chdir to $self->{tardir} ($!)";
 
-    find(sub {
-              my $entry = $File::Find::name;
-              $entry =~ s#^\./##;
-              my $type = (-d $_ ? "d" :
-                          -l $_ ? "l" :
-                                  "f"
-                         );
-              print FILE "$type $entry\n";
-            }, ".");
+    find(
+        sub {
+            my $entry = $File::Find::name;
+            $entry =~ s#^\./##;
+            my $type = (
+                  -d $_ ? "d"
+                : -l $_ ? "l"
+                :         "f"
+            );
+            print FILE "$type $entry\n";
+        },
+        "."
+    );
 
     chdir $cwd or LOGDIE "Can't chdir to $cwd ($!)";
 
@@ -309,41 +347,44 @@ sub list_reset {
 ###########################################
 sub list_next {
 ###########################################
-    my($self) = @_;
+    my ($self) = @_;
 
     my $offset = $self->offset();
 
-    my $list_file = File::Spec->catfile($self->{objdir}, "list");
+    my $list_file = File::Spec->catfile( $self->{objdir}, "list" );
     open FILE, "<$list_file" or LOGDIE "Can't open $list_file";
     seek FILE, $offset, 0;
 
-    { my $line = <FILE>;
+    {
+        my $line = <FILE>;
 
-      return undef unless defined $line;
+        return undef unless defined $line;
 
-      chomp $line;
-      my($type, $entry) = split / /, $line, 2;
-      redo if $type eq "d" and ! $self->{dirs};
-      $self->offset(tell FILE);
-      return [$entry, File::Spec->catfile($self->{tardir}, $entry), 
-              $type];
+        chomp $line;
+        my ( $type, $entry ) = split / /, $line, 2;
+        redo if $type eq "d" and !$self->{dirs};
+        $self->offset( tell FILE );
+        return [ $entry, File::Spec->catfile( $self->{tardir}, $entry ),
+            $type ];
     }
 }
 
 ###########################################
 sub offset {
 ###########################################
-    my($self, $new_offset) = @_;
+    my ( $self, $new_offset ) = @_;
 
-    my $offset_file = File::Spec->catfile($self->{objdir}, "offset");
+    my $offset_file = File::Spec->catfile( $self->{objdir}, "offset" );
 
-    if(defined $new_offset) {
+    if ( defined $new_offset ) {
         open FILE, ">$offset_file" or LOGDIE "Can't open $offset_file";
         print FILE "$new_offset\n";
         close FILE;
     }
 
-    open FILE, "<$offset_file" or LOGDIE "Can't open $offset_file (Did you call list_next() without a previous list_reset()?)";
+    open FILE, "<$offset_file"
+      or LOGDIE
+"Can't open $offset_file (Did you call list_next() without a previous list_reset()?)";
     my $offset = <FILE>;
     chomp $offset;
     return $offset;
@@ -353,13 +394,13 @@ sub offset {
 ###########################################
 sub write {
 ###########################################
-    my($self, $tarfile, $compress) = @_;
+    my ( $self, $tarfile, $compress ) = @_;
 
     my $cwd = getcwd();
     chdir $self->{tardir} or LOGDIE "Can't chdir to $self->{tardir} ($!)";
 
-    unless(File::Spec::Functions::file_name_is_absolute($tarfile)) {
-        $tarfile = File::Spec::Functions::rel2abs($tarfile, $cwd);
+    unless ( File::Spec::Functions::file_name_is_absolute($tarfile) ) {
+        $tarfile = File::Spec::Functions::rel2abs( $tarfile, $cwd );
     }
 
     my $compr_opt = "";
@@ -369,29 +410,32 @@ sub write {
     my @top_entries = grep { $_ !~ /^\.\.?$/ } readdir DIR;
     closedir DIR;
 
-    my $cmd = [$self->{tar}, "${compr_opt}cf$self->{tar_write_options}",
-               $tarfile, @{$self->{tar_gnu_write_options}}];
+    my $cmd = [
+        $self->{tar}, "${compr_opt}cf$self->{tar_write_options}",
+        $tarfile,     @{ $self->{tar_gnu_write_options} }
+    ];
 
-    if(@top_entries > $self->{max_cmd_line_args}) {
-        my $filelist_file = $self->{tmpdir}."/file-list";
-        open FLIST, ">$filelist_file" or 
-            LOGDIE "Cannot open $filelist_file ($!)";
-        for(@top_entries) {
+    if ( @top_entries > $self->{max_cmd_line_args} ) {
+        my $filelist_file = $self->{tmpdir} . "/file-list";
+        open FLIST, ">$filelist_file"
+          or LOGDIE "Cannot open $filelist_file ($!)";
+        for (@top_entries) {
             print FLIST "$_\n";
         }
         close FLIST;
         push @$cmd, "-T", $filelist_file;
-    } else {
+    }
+    else {
         push @$cmd, @top_entries;
     }
 
     DEBUG "Running @$cmd";
-    my $rc = run($cmd, \my($in, $out, $err));
+    my $rc = run( $cmd, \my ( $in, $out, $err ) );
 
-    if(!$rc) {
-         ERROR "@$cmd failed: $err";
-         chdir $cwd or LOGDIE "Cannot chdir to $cwd";
-         return undef;
+    if ( !$rc ) {
+        ERROR "@$cmd failed: $err";
+        chdir $cwd or LOGDIE "Cannot chdir to $cwd";
+        return undef;
     }
 
     WARN $err if $err;
@@ -404,87 +448,97 @@ sub write {
 ###########################################
 sub DESTROY {
 ###########################################
-    my($self) = @_;
+    my ($self) = @_;
 
-    $self->ramdisk_unmount() if defined  $self->{ramdisk};
+    $self->ramdisk_unmount() if defined $self->{ramdisk};
 
-    rmtree($self->{objdir}) if defined $self->{objdir};
-    rmtree($self->{tmpdir}) if defined $self->{tmpdir};
+    rmtree( $self->{objdir} ) if defined $self->{objdir};
+    rmtree( $self->{tmpdir} ) if defined $self->{tmpdir};
 }
 
 ###########################################
 sub is_gnu {
 ###########################################
-    my($self) = @_;
+    my ($self) = @_;
+    my ( $wtr, $rdr, $err ) = ( gensym, gensym, gensym );
+    my $pid = open3( $wtr, $rdr, $err, "$self->{tar} --version" );
+    my $output = join "\n", <$rdr>;
+    my $error  = join "\n", <$err>;
+    close($rdr);
+    close($err);
+    close($wtr);
+    waitpid( $pid, 0 );
+    my $exit = $? >> 8;
 
-    open PIPE, "$self->{tar} --version |" or 
+    unless ( $exit == 0 ) {
+        $self->{tar_error_msg} = $error;
         return 0;
-
-    my $output = join "\n", <PIPE>;
-    close PIPE;
-
-    return $output =~ /GNU/;
+    }
+    else {
+        return $output =~ /GNU/;
+    }
 }
 
 ###########################################
 sub ramdisk_mount {
 ###########################################
-    my($self, %options) = @_;
+    my ( $self, %options ) = @_;
 
-      # mkdir -p /mnt/myramdisk
-      # mount -t tmpfs -o size=20m tmpfs /mnt/myramdisk
+    # mkdir -p /mnt/myramdisk
+    # mount -t tmpfs -o size=20m tmpfs /mnt/myramdisk
 
-     $self->{mount}  = which("mount") unless $self->{mount};
-     $self->{umount} = which("umount") unless $self->{umount};
+    $self->{mount}  = which("mount")  unless $self->{mount};
+    $self->{umount} = which("umount") unless $self->{umount};
 
-     for (qw(mount umount)) {
-         if(!defined $self->{$_}) {
-             LOGWARN "No $_ command found in PATH";
-             return undef;
-         }
-     }
+    for (qw(mount umount)) {
+        if ( !defined $self->{$_} ) {
+            LOGWARN "No $_ command found in PATH";
+            return undef;
+        }
+    }
 
-     $self->{ramdisk} = { %options };
- 
-     $self->{ramdisk}->{size} = "100m" unless 
-       defined $self->{ramdisk}->{size};
- 
-     if(! defined $self->{ramdisk}->{tmpdir}) {
-         $self->{ramdisk}->{tmpdir} = tempdir( CLEANUP => 1 );
-     }
- 
-     my @cmd = ($self->{mount}, 
-                "-t", "tmpfs", "-o", "size=$self->{ramdisk}->{size}",
-                "tmpfs", $self->{ramdisk}->{tmpdir});
+    $self->{ramdisk} = {%options};
 
-     INFO "Mounting ramdisk: @cmd";
-     my $rc = system( @cmd );
- 
-    if($rc) {
+    $self->{ramdisk}->{size} = "100m"
+      unless defined $self->{ramdisk}->{size};
+
+    if ( !defined $self->{ramdisk}->{tmpdir} ) {
+        $self->{ramdisk}->{tmpdir} = tempdir( CLEANUP => 1 );
+    }
+
+    my @cmd = (
+        $self->{mount}, "-t", "tmpfs", "-o", "size=$self->{ramdisk}->{size}",
+        "tmpfs", $self->{ramdisk}->{tmpdir}
+    );
+
+    INFO "Mounting ramdisk: @cmd";
+    my $rc = system(@cmd);
+
+    if ($rc) {
         LOGWARN "Mount command '@cmd' failed: $?";
         LOGWARN "Note that this only works on Linux and as root";
         return;
     }
- 
+
     $self->{ramdisk}->{mounted} = 1;
- 
+
     return 1;
 }
 
 ###########################################
 sub ramdisk_unmount {
 ###########################################
-    my($self) = @_;
+    my ($self) = @_;
 
     return if !exists $self->{ramdisk}->{mounted};
 
-    my @cmd = ($self->{umount}, $self->{ramdisk}->{tmpdir});
+    my @cmd = ( $self->{umount}, $self->{ramdisk}->{tmpdir} );
 
     INFO "Unmounting ramdisk: @cmd";
 
-    my $rc = system( @cmd );
-        
-    if($rc) {
+    my $rc = system(@cmd);
+
+    if ($rc) {
         LOGWARN "Unmount command '@cmd' failed: $?";
         return;
     }
@@ -842,3 +896,7 @@ modify it under the same terms as Perl itself.
 =head1 AUTHOR
 
 2005, Mike Schilli <cpan@perlmeister.com>
+
+=head1 MAINTAINER
+
+2018, Alceu Rodrigues de Freitas Junior <arfreitas@cpan.org>
